﻿-- A set of different-looking buff bars. Inspired by Elkano's Buff Bars.
-- The player can left-click on a buff to cancel it.

-- Create our context so we can do rendering.
local context = UI.CreateContext("Context")

-- Because it's currently impossible to destroy frames, we keep a reservoir of buff bars around. Buff bars work as a single unified whole, so once they're created, we can just call their "set" function.
local activeBuffBars = {}
local zombieBuffBars = {}



local GrantBuff = {
-- TEST
--"Vitalité de l'Âme",
--"Tueur de créatures",

-- GSB : Duke Letareus
-- Rien de récupérable en debuff

-- GSB : Infiltrateur Johlen
-- Rien de récupérable en debuff

-- GSB : Oracle Aleria
"Necrotic Energy",
"Éruption nécrotique",

-- GSB : Prince Hylas
"Spirit Fracture",
"Fracture de l'âme",
"Contagious Cottontail",
"Queue de coton contagieuse",

-- GSB : Lord Greenscale
"Lashing Vines",
"Plante étrangleuse",
"Lianes cinglantes",
"Noxious Spores",
"Fougère nocive",
"Spores nocifs",

-- ROS : Maître de guerre Galenir
"Disturbing Howl",
"Hurlement perturbant",
"Howl of Death",
"Hurlement de Mort",
"Discordant Contagion",
"Contagion discordante",

-- ROS : Herault Gaurath
"Tidings of Woe",
"Nouvelles du Malheur",
"Aura of the Void",
"Aura du Néant",

-- ROS : Plutonus
"Walking Sleep",
"Somnambulisme",
"Shocking Cipher",
"Glyphe de choc",

-- ROS : Alsbeth
"Grave Bolt",
"Trait tombal",
"Punish Soul",
"Punir l'Âme",
"Discordant Meteor",
"Météore discordant",
"Discordant Ground",
"Sol Discordant",
"Explosion discordante"
}


local lastAttach = nil  -- Attachment point for our next buff bar.
local function StartBuffBars()  -- Hide everything we have and start adding bars one at a time.
  for _, v in pairs(activeBuffBars) do
    table.insert(zombieBuffBars, v)
    v:SetVisible(false)
    v:SetPoint("TOPCENTER", UIParent, "TOPCENTER")
  end
  activeBuffBars = {}
  lastAttach = nil
end


local function AddBuffBar(buff)
  bar = table.remove(zombieBuffBars)
  if not bar then
    -- We don't have any bars remaining, so we create a new one.
    -- Our Bars are considered single objects that can be dealt with atomically. Each one has the functionality needed to update itself.
    bar = UI.CreateFrame("Frame", "Bar", context)
    
    bar.text = UI.CreateFrame("Text", "Text", bar)
    bar.timer = UI.CreateFrame("Text", "Timer", bar)
    
    -- Solid background - this is the actual "bar" part of it.
    bar.solid = UI.CreateFrame("Frame", "Solid", bar)
    bar.solid:SetLayer(-1)  -- Put it behind every other element.
    
    bar.text:SetText("")
    bar.text:SetHeight(bar.text:GetFullHeight())  -- Set an initial height. We'll be overriding this, we just want to make sure it works.
    
    bar.text:SetPoint("TOPLEFT", bar, "TOPLEFT") -- The text is pinned to the top-left corner of the bar.
    bar:SetPoint("BOTTOM", bar.text, "BOTTOM")  -- The bar is set to always be as high as the text is.
    
    bar.timer:SetPoint("TOPRIGHT", bar, "TOPRIGHT") -- The timer is pinned to the top-right corner.
    
    bar.text:SetPoint("RIGHT", bar.timer, "LEFT") -- Make sure the text doesn't overrun the timer. We'll be changing the text's height based on the contents, but we'll leave the width calculated this way.
    
    -- Set the solid bar to fill the entire buff bar.
    bar.solid:SetPoint("TOPLEFT", bar, "TOPLEFT")
    bar.solid:SetPoint("BOTTOMRIGHT", bar, "BOTTOMRIGHT")
    
    -- This is hardcoded, but in a full fleshed-out addon, it would be set by the user.
    -- This could be done now with slash commands, but couldn't be saved yet.
    bar:SetWidth(320)
    -- The function that actually sets everything.
    function bar:SetBuff(buff)
	
			if buff.debuff then
				-- For the sake of this addon, Debuffs are somewhat larger and red-themed.
				self:SetBackgroundColor(0.5, 0.3, 0.3, 0.3)
				self.solid:SetBackgroundColor(0.5, 0.3, 0.3)
				self.text:SetFontSize(20)
				self.text:SetHeight(self.text:GetFullHeight())
			  else
				-- Buffs are smaller and blue-themed.
				self:SetBackgroundColor(0.2, 0.2, 0.6, 0.3)
				self.solid:SetBackgroundColor(0.2, 0.2, 0.6)
				self.text:SetFontSize(12)
				self.text:SetHeight(self.text:GetFullHeight())
			  end
			  
			  --print_console (GrantBuff[0])
			  
			  -- Display our stacking multiple.
			  if buff.stack then
				self.text:SetText(buff.name .. " x" .. buff.stack)
			  else
				self.text:SetText(buff.name)
			  end
			  
			  if buff.duration then
				-- Display our timer.
				self.solid:SetPoint("RIGHT", bar, buff.remaining / buff.duration, nil)
				-- Rift's UI system allows you to pin points to an arbitrary place on a frame, not just the corners and edges.
				-- These two lines are equivalent:
				-- a:SetPoint("TOPLEFT", b, "BOTTOMLEFT")
				-- a:SetPoint(0, 0, b, 0, 1)
				-- Coordinates are passed in X, Y order, with 0 being the topleft and 1 being the bottomright.
				-- We're using this feature to make Rift do the position calculation for us.
				
				-- Generate the timer text string.
				if buff.remaining >= 3600 then
				  self.timer:SetText(string.format("%d:%02d:%02d", buff.remaining / 3600, buff.remaining / 60 % 60, buff.remaining % 60))
				else
				  self.timer:SetText(string.format("%d:%02d", buff.remaining / 60, buff.remaining % 60))
				end
				
				-- Update the width to avoid truncation.
				self.timer:SetWidth(self.timer:GetFullWidth())
				
				-- Display everything we might have hidden.
				self.solid:SetVisible(true)
				self.timer:SetVisible(true)
			  else
				-- This is a permanent buff without a timer, so don't show any of that.
				self.solid:SetVisible(false)
				self.timer:SetVisible(false)
				self.timer:SetWidth(0)
			  end
		
		
		self.buff = buff.buff
		self.buffid = buff.id
    end
    
    -- Finally, if we're clicked, we want to cancel whatever buff is on us.
    function bar.Event:LeftDown()
      Command.Buff.Cancel(self.buffid)
    end
  end
  
  table.insert(activeBuffBars, bar)
  
  -- Show the bar and set the data.
  bar:SetVisible(true)
  bar:SetBuff(buff)
  
  -- Attach it to the right spot.
  if not lastAttach then
    -- This is our first bar, so we're pinning it to the top center, slightly below the top edge of the screen. Most of the screen is taken by the standard UI and this is the least cluttered place we have.
    -- The downside is that lots of buffs will run down right into the player's field of view.
    bar:SetPoint("TOPCENTER", UIParent, "TOPCENTER", 0, 50)
  else
    -- This isn't our first bar, so we pin it to the last bar.
    if lastAttach.buff ~= buff.buff then
      -- We're the first debuff, so add a gap between us and the last bar.
      bar:SetPoint("TOPCENTER", lastAttach, "BOTTOMCENTER", 0, 20)
    else
      -- Otherwise, we're flush with the last bar.
      bar:SetPoint("TOPCENTER", lastAttach, "BOTTOMCENTER")
    end
  end
  lastAttach = bar
end

-- Rift does not currently have buff update events, so we're redrawing the entire set of buff bars every frame. This is very slow, and we're aware of it, and we'll have buff update events soon. :)

local function refresh()

	if badbossactive ~= "SHOW" then return end
	  -- Get the list of buffs from the player. Note that "player" can be replaced by "player.target" to turn this into a target's-buff-bar display.
	  local bufflist = Inspect.Buff.List("player")
	  if not bufflist then return end -- If we don't get anything, then we don't currently have information about the player. This may happen when the player is logging in or teleporting long distances.
	  
	  local buffdetails = Inspect.Buff.Detail("player", bufflist) -- Retrieve the details for all of our buffs. If there were only certain buffs we cared about, we could filter them out before asking for detail.
	  
	  StartBuffBars() -- Reset the buff bars and start entering buffs.
	  
	  -- We want to order buffs by their time remaining, splitting apart buffs and debuffs. We do the ordering here, in standard Lua code.
	  local bbars = {}
	  for id, buff in pairs(buffdetails) do
		if buff.debuff then
			for _,v in pairs(GrantBuff) do
			  
			  if v == buff.name then
				buff.id = id  -- Make a copy of the ID, because we'll need it
				table.insert(bbars, buff)
				
			 end
			  
			end -- verif buff granted
		
		end

	  end
	  
	  table.sort(bbars, function (a, b)
		if a.buff ~= b.buff then
		  return not b.buff
		end
		
		if a.duration and b.duration then return a.remaining > b.remaining end
		if not a.duration and not b.duration then return false end
		return not a.duration
	  end)
	  
	  -- Now that we have the ordering, we just add the bars one at a time. Done!
	  for k, v in ipairs(bbars) do
		AddBuffBar(v)
	  end
	
end

local function badbossSHOW()

bar:SetVisible(true)
badbossactive = "SHOW"
print("BadBossPVE is now " .. badbossactive)

end

local function badbossHIDE()

bar:SetVisible(false)
badbossactive = "HIDE"
print("BadBossPVE is now " .. badbossactive)


end

local function badbossHELP()

print("BadBossPVE Version : " .. badbossversion)
print("Status : " .. badbossactive)
print("=======================")
print("SHOW ALERT : /bbshow")
print("HIDE ALERT : /bbhide")
print("HELP : /bb")


end

--Register commands
table.insert(Command.Slash.Register("bbshow"), {function (params)
	badbossSHOW()
	
end, "BadBossPVE", "Slash command"})

table.insert(Command.Slash.Register("bbhide"), {function (params)
	badbossHIDE()
end, "BadBossPVE", "Slash command"})

table.insert(Command.Slash.Register("bb"), {function (params)
	badbossHELP()
end, "BadBossPVE", "Slash command"})

-- Here's our event. As mentioned, this triggers every single frame. See TrionDevelopmentTools for more detail on events.
badbossactive = "SHOW"
badbossversion = "0.1 alpha"
badbossHELP()
table.insert(Event.System.Update.Begin, {refresh, "BadBossPVE", "refresh"})



